*
* Clk_7seg.asm  Real time clock for the Ep2711E9 Rev. C board
*               (c)2003, EVBplus.com, Written By Wayne Chu
*
*     Function:	This program initially reads the time from the host PC
*		system clock instead of a real time clock chip, then
*		shows the current time in HH:MM or MM:SS format
*		on the seven segment LED display. It will work without
*		the host PC after the initial reading.
*
*		The default format is HH:MM.  When the PA0 SW is depressed
*		the fromat is changed to MM:SS. The AM/PM is indicated 
*		by the decimal point of the rightmost digit (digit 0).
*
*		The program has a minor bug, it does not read the PC
*		clock correctly at the first time. In order to run the
*		program, you enter go\start, then press the ESC key to stop
*		the program, then enter go \start again. the second go \start
*		will work.  We think that a smart student may be able to
*		help us to find the bug. 
*
* 		All EVB/EVBU I/O routines are located at $0FA0-$0FCF
*		in stead of $FFA0-$FFCF.
*
*
              	org     $0800	; EVBplus board I/O routines
rs485_recv:     rmb     3	; enables rs485 recv mode
rs485_xmit:     rmb     3	; enables rs485 xmit mode
get_date:       rmb     3	; gets date info. from host PC
get_time:       rmb     3	; gets time info. from host PC
outstrg00:	rmb	3	; print a string ended by 0
lcd_ini:	rmb	3	; initializes the 16x2 LCD module
lcd_line1:	rmb	3	; displays 16 char on the first line
lcd_line2:	rmb	3	; displays 16 char on the second line
sel_inst:	rmb	3	; selects instruction before writing LCD module 
sel_data:	rmb	3	; selects data before writing the LCD module
wrt_pulse:	rmb	3	; generates a write pulse to the LCD module
seven_segment:  rmb	3	; convert Accu A to segment pattern, bit 7= DP
*
* Segment conversion table in the monitor firmware:

* Binary number:	 	0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
* Converted to segment code:  	0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
*
* Binary number:		$10,$11,$12,$13,$14,$15,$16,$17
* Converted to segment code:     G   H   h   J   L   n   o   o
*
* Binary number:  		$18,$19,$1A,$1B,$1C,$1D,$1E,$1F,$20
* Converted to segment code:     P   r   t   U   u   y   _  --  Blank
*
                org     $0FA0	; Buffalo I/O routines are duplicated here
upcase:         rmb     3
wchek:          rmb     3
dchek:          rmb     3
init_sci:       rmb     3
input:          rmb     3
output:         rmb     3
outlhlf:        rmb     3
outrhlf:        rmb     3
outa:           rmb     3
out1byt:        rmb     3
out1bsp:        rmb     3
out2bsp:        rmb     3
outcrlf:        rmb     3
outstrg:        rmb     3
outstrg0:	rmb	3
inchar:         rmb     3
*
*
DB0:		equ	1
DB1:		equ	2
DB2:		equ   	4
DB3:		equ   	8
DB4:		equ	$10
DB5:		equ	$20
DB6:		equ	$40
DB7:		equ	$80
NOTDB6:		equ	$BF
portb:		equ	4
portd:		equ	8
ddrd:		equ	9
toc2:		equ	$18	
tctl1:		equ	$20
tmsk1:		equ	$22
tflg1:		equ	$23	
tmsk2:		equ	$24	

REGBLK:		equ	$1000
TB5MS:		equ	10000	; time base of 10,000 instruction cycles

ONE_SEC:	equ	200
HALF_SEC:	equ	100
*
*
		org	$40
ram:		rmb	1
pctime:		rmb	12	; 11 time chars(hh:mm:ss pm) + end character
hour10:		rmb	1
hour:		rmb	1
min10:		rmb	1
min:		rmb	1
sec10:		rmb	1
sec:		rmb	1
time_image:	rmb	6
amfm_flg:	rmb	1
loopcnt:	rmb	1		; 5 ms/loop	
*
select:		rmb	1
d5ms_flag:	rmb	1
disp_data:	rmb	4
disptn:		rmb	4
*
ramend:		rmb	1
ALLRAM:		equ 	ramend-ram      ;total ram used
*
STACK:		equ	$FF
*
*
	org	$F000
	jmp	start
*
clrall:	pshx
	ldx	#ram
clrr	clr	0,x
	inx
	deca
	bne	clrr
	pulx
	rts
*
start:
	lds	#STACK
	ldx	#REGBLK
*	ldaa	#11111111b
	ldaa	#$ff
	staa	portd,x
*	ldaa 	#11111100b	; 0 =input, 1 = output
	ldaa 	#$fc
	staa 	ddrd,x		
*
	ldx	#REGBLK
	ldaa	#DB6	 
	staa	tmsk1,x

	ldaa	#ALLRAM
	jsr	clrall		; clear all ram
	cli

bad_format:
	ldx	#pctime
	jsr	get_time	; get the 11 character time string 
*                               ; from pc's clock and it will be stored 
*				; at the ram locations (pctime to pctime+10) 
*				; 11 time chars(hh:mm:ss PM) + end character
	ldx	#pctime
	ldaa	2,x
	cmpa	#':'
	bne	bad_format
	ldaa	5,x
	cmpa	#':'
	bne	bad_format
	ldaa	8,x
	cmpa	#' '
	bne	bad_format
	ldaa	10,x
	cmpa	#'M'
	bne	bad_format
*
* good data
	ldab	0,x
	stab	hour10
	ldab	1,x
	stab	hour
*
	ldab	3,x
	stab	min10
	ldab	4,x
	stab	min
	ldab	6,x
	stab	sec10
	ldab	7,x
	stab	sec
	ldab	9,x
	cli
*
begin:	
	inc	loopcnt
	ldaa	loopcnt
	cmpa	#ONE_SEC
	bne	no_update	; not one second yet
	
	clr	loopcnt
	inc	sec
	ldaa	sec
	cmpa	#'9'
	ble	no_update	

	ldaa	#'0'
	staa	sec
	inc	sec10
	ldaa	sec10
	cmpa	#'5'	
	ble	no_update	

	ldaa	#'0'
	staa	sec10
	inc	min
	ldaa	min
	cmpa	#'9'	
	ble	no_update	

	ldaa	#'0'
	staa	min
	inc	min10
	ldaa	min10
	cmpa	#'5'
	ble	no_update	

	ldaa	#'0'
	staa	min10
	inc	hour
	ldaa	hour
	cmpa	#'9'
	ble	no_update	
	ldaa	#'0'
	staa	hour
	inc	hour10
*
no_update:
	ldx	hour10
	cpx	#$3132		; compare if time = '1' '2'
	bne	test13		; if not, test if time = '1' '3'
	ldaa	amfm_flg
	bne	test13		; bypass if time still = '1 
	inc	amfm_flg	; am and pm are checked already
*
	ldaa	pctime+9	; AM/PM byte
	cmpa	#'A'
	beq	make_to_pm
	ldaa	#'A'
	staa	pctime+9	; AM/PM byte
	jmp	dspl_out
make_to_pm:
	ldaa	#'P'
	staa	pctime+9	; AM/PM byte
	jmp	dspl_out
*
test13:	ldx	hour10
	cpx	#$3133		; compare if time = '1' '3'
	bne	dspl_out	; if not bypass
	clr	amfm_flg
	ldx	#$3031		; if yes, change to one o'clock
	stx	hour10
*
dspl_out:
	ldx	#hour10
	ldy	#time_image
copy_next:
	ldaa	0,x
	staa	0,y
	inx
	iny
	cpx	#hour10+6
	bne	copy_next
*
	ldx	#time_image
	ldaa	0,x		; check leading zero	
	cmpa	#'0'		; if it's a zero, change it to 
	bne	out		; the blank character
	adda	#$20		; #$20 will be changed to 
	staa	0,x		; the blank char by the move_asc subroutine			; subroutine
out:	nop

 	ldx	#time_image+2	; display clock in mm:ss format
        ldy     #REGBLK         ; x register points to register block
	brclr   0,y $01 mm_ss   ; if PA0 switch is down, keeps mm:ss format
	
	ldx	#time_image	; display clock in hh:mm format
mm_ss:	jsr	move_asc
	jsr	flash_dp	; flash decimal point
	jsr	add_am_pm	; turn on the dp of the digit0 if it's PM
	jsr	sel_digit

*
wait:	tst	d5ms_flag
	beq	wait
	clr	d5ms_flag
	jmp	begin

flash_dp:
	ldaa	loopcnt
	cmpa	#HALF_SEC
	bcs	dp_on		; decimal point is on for 1/2 second
	ldaa	disptn+1
	anda	#$7F		; decimal point is off
	staa	disptn+1
	ldaa	disptn+2
	anda	#$7F		; decimal point is off
	staa	disptn+2
	rts
dp_on:	ldaa	disptn+1
	oraa	#$80		; decimal point is on
	staa	disptn+1
	ldaa	disptn+2
	oraa	#$80		; decimal point is on
	staa	disptn+2
	rts
*
add_am_pm:
	ldaa	pctime+9	; AM/PM byte
	cmpa	#'A'
	beq	am
	ldaa	disptn+3
	oraa	#$80		; dot is on = pm
	staa	disptn+3
	rts
am:	ldaa	disptn+3
	anda	#$7F		; dot is off =am
	staa	disptn+3
	rts
*
*    this routine moves 4 bytes of data into display
*    pattern and converts the pattern to seven segment code.
*    @ enter, x points the source address
*
move_asc:
	ldy	#disptn
mnext: 	ldaa	0,x
	suba	#'0'		; change it to binary number
	jsr	seven_segment	; convert Accu A to segment pattern, bit 7= DP
	staa	0,y
	inx
	iny
	cpy	#disptn+4
        bne	mnext
	rts
*
* multiplexing display one digit at a time
*
sel_digit:
	ldx	#REGBLK
	inc	select
	ldab	select
	andb	#3
	tstb
	beq	digit3
	decb
	beq	digit2
	decb
	beq	digit1
*
digit0: 
	ldaa	disptn+3
	staa	portb,x
	bclr	portd,x 4		; turn on digit 0
	bset	portd,x 8		; turn off all other digits
	bset	portd,x $10
	bset	portd,x $20
	rts
digit1: 
	ldaa	disptn+2
	staa	portb,x
	bset	portd,x 4
	bclr	portd,x 8		; turn on digit 1
	bset	portd,x $10		; turn off all other digits
	bset	portd,x $20
	rts
digit2: 
	ldaa	disptn+1
	staa	portb,x
	bset	portd,x 4
	bset	portd,x 8
	bclr	portd,x $10		; turn on digit 2
	bset	portd,x $20
	rts
digit3: 
	ldaa	disptn
	staa	portb,x
	bset	portd,x 4
	bset	portd,x 8
	bset	portd,x $10
	bclr	portd,x $20		; turn on digit 3
	rts
tmr6oc2:
	ldx	#REGBLK	
* in an interrupt servicing routine the x register will be saved automatically
* the rti instruction will pop the x register off stack.
	inc	d5ms_flag
	ldd	#TB5MS		; reload the count for 5 ms time base
	addd	toc2,x
	std	toc2,x
	bclr	tflg1,x NOTDB6
	rti
*
	org	$FFE6
	fdb	tmr6oc2
	org	$FFFE
	fdb	start
	end

